@@ -14,9 +14,15 @@
 
 #include "sentencepiece_processor.h"
 
+#include <algorithm>
+#include <cmath>
+#include <cstddef>
+#include <iterator>
 #include <map>
+#include <memory>
 #include <set>
 #include <utility>
+#include <vector>
 
 #include "common.h"
 #include "filesystem.h"
@@ -24,7 +30,6 @@
 #include "model_interface.h"
 #include "normalizer.h"
 #include "sentencepiece.pb.h"
-#include "third_party/absl/memory/memory.h"
 #include "third_party/absl/strings/numbers.h"
 #include "third_party/absl/strings/str_cat.h"
 #include "third_party/absl/strings/str_join.h"
@@ -211,7 +216,7 @@ SentencePieceProcessor::SentencePieceProcessor() {}
 SentencePieceProcessor::~SentencePieceProcessor() {}
 
 util::Status SentencePieceProcessor::Load(absl::string_view filename) {
-  auto model_proto = absl::make_unique<ModelProto>();
+  auto model_proto = std::make_unique<ModelProto>();
   RETURN_IF_ERROR(io::LoadModelProto(filename, model_proto.get()));
   return Load(std::move(model_proto));
 }
@@ -221,14 +226,14 @@ void SentencePieceProcessor::LoadOrDie(absl::string_view filename) {
 }
 
 util::Status SentencePieceProcessor::Load(const ModelProto &model_proto) {
-  auto model_proto_copy = absl::make_unique<ModelProto>();
+  auto model_proto_copy = std::make_unique<ModelProto>();
   *model_proto_copy = model_proto;
   return Load(std::move(model_proto_copy));
 }
 
 util::Status SentencePieceProcessor::LoadFromSerializedProto(
     absl::string_view serialized) {
-  auto model_proto = absl::make_unique<ModelProto>();
+  auto model_proto = std::make_unique<ModelProto>();
   CHECK_OR_RETURN(
       model_proto->ParseFromArray(serialized.data(), serialized.size()));
   return Load(std::move(model_proto));
@@ -238,11 +243,11 @@ util::Status SentencePieceProcessor::Load(
     std::unique_ptr<ModelProto> model_proto) {
   model_proto_ = std::move(model_proto);
   model_ = ModelFactory::Create(*model_proto_);
-  normalizer_ = absl::make_unique<normalizer::Normalizer>(
+  normalizer_ = std::make_unique<normalizer::Normalizer>(
       model_proto_->normalizer_spec(), model_proto_->trainer_spec());
   if (model_proto_->has_denormalizer_spec() &&
       !model_proto_->denormalizer_spec().precompiled_charsmap().empty()) {
-    denormalizer_ = absl::make_unique<normalizer::Normalizer>(
+    denormalizer_ = std::make_unique<normalizer::Normalizer>(
         model_proto_->denormalizer_spec());
   }
 
@@ -409,7 +414,7 @@ util::Status SentencePieceProcessor::Decode(
 
   SentencePieceText spt;
   RETURN_IF_ERROR(Decode(pieces, &spt));
-  *detokenized = std::move(spt.text());
+  *detokenized = std::move(*spt.mutable_text());
 
   return util::OkStatus();
 }
@@ -420,7 +425,7 @@ util::Status SentencePieceProcessor::Decode(const std::vector<int> &ids,
 
   SentencePieceText spt;
   RETURN_IF_ERROR(Decode(ids, &spt));
-  *detokenized = std::move(spt.text());
+  *detokenized = std::move(*spt.mutable_text());
 
   return util::OkStatus();
 }
@@ -623,10 +628,10 @@ util::Status SentencePieceProcessor::PopulateSentencePieceText(
   CHECK_EQ_OR_RETURN(consumed, normalized.size())
       << "all normalized characters are not consumed.";
 
-  RETURN_IF_ERROR(ApplyExtraOptions(encode_extra_options_, spt));
-
   spt->set_text(input.data(), input.size());
 
+  RETURN_IF_ERROR(ApplyExtraOptions(encode_extra_options_, spt));
+
   return util::OkStatus();
 }  // namespace sentencepiece
 
@@ -695,10 +700,17 @@ util::Status SentencePieceProcessor::SampleEncode(
     const auto nbests = model_->NBestEncode(normalized, nbest_size);
     CHECK_OR_RETURN(!nbests.empty()) << "NBestEncode returns empty result.";
 
-    std::vector<float> probs(nbests.size(), 0.0);
-    for (size_t i = 0; i < nbests.size(); ++i) {
-      probs[i] = std::exp(alpha * nbests[i].second);
-    }
+    std::vector<double> log_probs;
+    log_probs.reserve(nbests.size());
+    std::transform(nbests.begin(), nbests.end(), std::back_inserter(log_probs),
+                   [alpha](const auto &nbest) { return alpha * nbest.second; });
+
+    const double Z = log_domain::LogSum(log_probs);
+    std::vector<double> probs;
+    probs.reserve(log_probs.size());
+    std::transform(
+        log_probs.begin(), log_probs.end(), std::back_inserter(probs),
+        [Z](const auto &log_prob) { return std::exp(log_prob - Z); });
 
     auto *mt = random::GetRandomGenerator();
     std::discrete_distribution<int> dist(probs.begin(), probs.end());
@@ -918,6 +930,26 @@ util::Status SentencePieceProcessor::Decode(const std::vector<int> &ids,
     return value;                                                            \
   }
 
+util::Status SentencePieceProcessor::Normalize(absl::string_view input,
+                                               std::string *normalized) const {
+  std::vector<size_t> norm_to_orig;
+  CHECK_OR_RETURN(normalizer_);
+  return normalizer_->Normalize(input, normalized, &norm_to_orig);
+}
+
+util::Status SentencePieceProcessor::Normalize(
+    absl::string_view input, std::string *normalized,
+    std::vector<size_t> *norm_to_orig) const {
+  CHECK_OR_RETURN(normalizer_);
+  return normalizer_->Normalize(input, normalized, norm_to_orig);
+}
+
+std::string SentencePieceProcessor::Normalize(absl::string_view input) const {
+  std::string normalized;
+  Normalize(input, &normalized).IgnoreError();
+  return normalized;
+}
+
 int SentencePieceProcessor::GetPieceSize() const {
   CHECK_STATUS_OR_RETURN_DEFAULT(0);
   return model_->GetPieceSize();
@@ -998,6 +1030,8 @@ util::Status SentencePieceProcessor::ApplyExtraOptions(
         piece->set_id(PieceToId(absl::string_view(model_->eos_piece().data())));
         piece->set_piece(model_->eos_piece().data(),
                          model_->eos_piece().size());
+        piece->set_begin(spt->text().size());
+        piece->set_end(spt->text().size());
       } break;
       case BOS: {
         auto *array = spt->mutable_pieces();
@@ -1009,6 +1043,8 @@ util::Status SentencePieceProcessor::ApplyExtraOptions(
         piece->set_id(PieceToId(absl::string_view(model_->bos_piece().data())));
         piece->set_piece(model_->bos_piece().data(),
                          model_->bos_piece().size());
+        piece->set_begin(0);
+        piece->set_end(0);
       } break;
       case UNK_PIECE: {
         for (int i = 0; i < spt->pieces_size(); ++i) {
@@ -1081,6 +1117,10 @@ std::string SentencePieceProcessor::serialized_model_proto() const {
   return model_proto_ ? model_proto_->SerializeAsString() : "";
 }
 
+NormalizerSpec *SentencePieceProcessor::mutable_normalizer_spec() const {
+  return model_proto_ ? model_proto_->mutable_normalizer_spec() : nullptr;
+}
+
 // Set seed value of random generator.
 // Do not set static_cast<unique_int>(-1),
 // as this seed is reserved for initializing from
@@ -1097,9 +1137,13 @@ util::Status LoadModelProto(absl::string_view filename,
   auto input = filesystem::NewReadableFile(filename, true);
   RETURN_IF_ERROR(input->status());
   std::string serialized;
-  CHECK_OR_RETURN(input->ReadAll(&serialized));
-  CHECK_OR_RETURN(
-      model_proto->ParseFromArray(serialized.data(), serialized.size()));
+  if (!input->ReadAll(&serialized)) {
+    return util::InternalError(absl::StrCat("could not read ", filename));
+  }
+  if (!model_proto->ParseFromArray(serialized.data(), serialized.size())) {
+    return util::InternalError(
+        absl::StrCat("could not parse ModelProto from ", filename));
+  }
 
   return util::OkStatus();
 }
