@@ -25,7 +25,6 @@
 #include "sentencepiece_trainer.h"
 #include "testharness.h"
 #include "third_party/absl/container/flat_hash_map.h"
-#include "third_party/absl/memory/memory.h"
 #include "third_party/absl/strings/str_cat.h"
 #include "third_party/absl/strings/string_view.h"
 #include "util.h"
@@ -123,7 +122,7 @@ NormalizerSpec MakeDefaultNormalizerSpec() {
 TEST(SentencepieceProcessorTest, StatusTest) {
   SentencePieceProcessor sp;
   EXPECT_FALSE(sp.status().ok());
-  auto mock = absl::make_unique<MockModel>();
+  auto mock = std::make_unique<MockModel>();
   sp.SetModel(std::move(mock));
   EXPECT_FALSE(sp.status().ok());
 }
@@ -135,7 +134,7 @@ TEST(SentencepieceProcessorTest, EncodeTest) {
   const auto normalization_spec = MakeDefaultNormalizerSpec();
 
   {
-    auto mock = absl::make_unique<MockModel>();
+    auto mock = std::make_unique<MockModel>();
 
     const EncodeResult result = {
         {WS "ABC", 3}, {WS "DE", 4}, {"F", 0}, {"</s>", 2}};
@@ -143,7 +142,7 @@ TEST(SentencepieceProcessorTest, EncodeTest) {
 
     sp.SetModel(std::move(mock));
     sp.SetNormalizer(
-        absl::make_unique<normalizer::Normalizer>(normalization_spec));
+        std::make_unique<normalizer::Normalizer>(normalization_spec));
 
     std::vector<std::string> output;
     EXPECT_TRUE(sp.Encode("ABC DEF", &output).ok());
@@ -186,7 +185,7 @@ TEST(SentencepieceProcessorTest, EncodeTest) {
 
   // Unknown sequences.
   {
-    auto mock = absl::make_unique<MockModel>();
+    auto mock = std::make_unique<MockModel>();
 
     const EncodeResult result = {
         {WS "ABC", 3}, {WS "D", 4}, {"E", 0}, {"F", 0}, {"</s>", 2}};
@@ -196,7 +195,7 @@ TEST(SentencepieceProcessorTest, EncodeTest) {
     mock->SetEncodeResult(kInput, result);
     sp.SetModel(std::move(mock));
     sp.SetNormalizer(
-        absl::make_unique<normalizer::Normalizer>(normalization_spec));
+        std::make_unique<normalizer::Normalizer>(normalization_spec));
 
     std::vector<std::string> output;
     EXPECT_TRUE(sp.Encode("ABC DEF", &output).ok());
@@ -236,7 +235,7 @@ TEST(SentencepieceProcessorTest, EncodeTest) {
   // Byte-fallback.
   {
     const absl::string_view kInput2 = WS "ABC" WS "DEFあ";
-    auto mock = absl::make_unique<ByteFallbackMockModel>();
+    auto mock = std::make_unique<ByteFallbackMockModel>();
 
     const EncodeResult result = {{WS "ABC", 3}, {WS "D", 4}, {"E", 0},
                                  {"F", 0},      {"あ", 0},   {"</s>", 2}};
@@ -250,7 +249,7 @@ TEST(SentencepieceProcessorTest, EncodeTest) {
     mock->SetEncodeResult(kInput2, result);
     sp.SetModel(std::move(mock));
     sp.SetNormalizer(
-        absl::make_unique<normalizer::Normalizer>(normalization_spec));
+        std::make_unique<normalizer::Normalizer>(normalization_spec));
 
     std::vector<std::string> output;
     EXPECT_TRUE(sp.Encode("ABC DEFあ", &output).ok());
@@ -306,12 +305,12 @@ TEST(SentencepieceProcessorTest, EncodeTest) {
   // Crash if
   // ModelInterface::Encode() returns shorter results.
   {
-    auto mock = absl::make_unique<MockModel>();
+    auto mock = std::make_unique<MockModel>();
     const EncodeResult result = {{WS "ABC", 3}};
     mock->SetEncodeResult(kInput, result);
     sp.SetModel(std::move(mock));
     sp.SetNormalizer(
-        absl::make_unique<normalizer::Normalizer>(normalization_spec));
+        std::make_unique<normalizer::Normalizer>(normalization_spec));
     SentencePieceText spt;
     // Expects crash.
     EXPECT_FALSE(sp.Encode("ABC DEF", &spt).ok());
@@ -320,13 +319,13 @@ TEST(SentencepieceProcessorTest, EncodeTest) {
   // Crash if
   // ModelInterface::Encode() returns longer results.
   {
-    auto mock = absl::make_unique<MockModel>();
+    auto mock = std::make_unique<MockModel>();
     const EncodeResult result = {
         {WS "ABC", 3}, {WS "DE", 4}, {"F", 5}, {"G", 6}};
     mock->SetEncodeResult(kInput, result);
     sp.SetModel(std::move(mock));
     sp.SetNormalizer(
-        absl::make_unique<normalizer::Normalizer>(normalization_spec));
+        std::make_unique<normalizer::Normalizer>(normalization_spec));
     SentencePieceText spt;
     // Expects crash.
     EXPECT_FALSE(sp.Encode("ABC DEF", &spt).ok());
@@ -335,13 +334,13 @@ TEST(SentencepieceProcessorTest, EncodeTest) {
   // Crash if
   // ModelInterface::Encode() returns an empty piece.
   {
-    auto mock = absl::make_unique<MockModel>();
+    auto mock = std::make_unique<MockModel>();
     const EncodeResult result = {
         {WS "ABC", 3}, {WS "DE", 4}, {"", 5}, {"F", 6}};
     mock->SetEncodeResult(kInput, result);
     sp.SetModel(std::move(mock));
     sp.SetNormalizer(
-        absl::make_unique<normalizer::Normalizer>(normalization_spec));
+        std::make_unique<normalizer::Normalizer>(normalization_spec));
     SentencePieceText spt;
     // Expects crash.
     EXPECT_FALSE(sp.Encode("ABC DEF", &spt).ok());
@@ -349,7 +348,7 @@ TEST(SentencepieceProcessorTest, EncodeTest) {
 
   // Halfwidth to Fullwidith katakana normalization.
   {
-    auto mock = absl::make_unique<MockModel>();
+    auto mock = std::make_unique<MockModel>();
     const EncodeResult result = {{WS "グー", 3}, {"グル", 4}, {"</s>", 2}};
     const absl::string_view input = WS "グーグル";
     mock->SetEncodeResult(input, result);
@@ -383,7 +382,7 @@ TEST(SentencepieceProcessorTest, EncodeTest) {
 
   // One to many normalization.
   {
-    auto mock = absl::make_unique<MockModel>();
+    auto mock = std::make_unique<MockModel>();
     const EncodeResult result = {{WS "株式", 3}, {"会社", 4}, {"</s>", 2}};
     const absl::string_view input = WS "株式会社";
     mock->SetEncodeResult(input, result);
@@ -422,7 +421,7 @@ TEST(SentencepieceProcessorTest, NBestEncodeTest) {
 
   const auto normalization_spec = MakeDefaultNormalizerSpec();
 
-  auto mock = absl::make_unique<MockModel>();
+  auto mock = std::make_unique<MockModel>();
 
   const NBestEncodeResult result = {
       {{{WS "ABC", 3}, {WS "DE", 4}, {"F", 0}, {"</s>", 2}},
@@ -433,7 +432,7 @@ TEST(SentencepieceProcessorTest, NBestEncodeTest) {
   mock->SetNBestEncodeResult(kInput, result);
   sp.SetModel(std::move(mock));
   sp.SetNormalizer(
-      absl::make_unique<normalizer::Normalizer>(normalization_spec));
+      std::make_unique<normalizer::Normalizer>(normalization_spec));
 
   std::vector<std::vector<std::string>> output;
   EXPECT_TRUE(sp.NBestEncode("ABC DEF", 2, &output).ok());
@@ -464,7 +463,7 @@ TEST(SentencepieceProcessorTest, NBestEncodeTest) {
       spt2.ParseFromString(sp.NBestEncodeAsSerializedProto("ABC DEF", 2)));
   EXPECT_EQ(spt.SerializeAsString(), spt2.SerializeAsString());
 
-  auto mock_empty = absl::make_unique<MockModel>();
+  auto mock_empty = std::make_unique<MockModel>();
   mock_empty->SetNBestEncodeResult(kInput, {});
   sp.SetModel(std::move(mock_empty));
   EXPECT_FALSE(sp.NBestEncode("ABC DEF", 2, &output).ok());
@@ -476,7 +475,7 @@ TEST(SentencepieceProcessorTest, SampleEncodeTest) {
 
   const auto normalization_spec = MakeDefaultNormalizerSpec();
 
-  auto mock = absl::make_unique<MockModel>();
+  auto mock = std::make_unique<MockModel>();
 
   const EncodeResult result = {
       {WS "ABC", 3}, {WS "DE", 4}, {"F", 0}, {"</s>", 2}};
@@ -490,7 +489,7 @@ TEST(SentencepieceProcessorTest, SampleEncodeTest) {
   mock->SetEncodeResult(kInput, result);
   sp.SetModel(std::move(mock));
   sp.SetNormalizer(
-      absl::make_unique<normalizer::Normalizer>(normalization_spec));
+      std::make_unique<normalizer::Normalizer>(normalization_spec));
 
   std::vector<std::string> output;
   EXPECT_TRUE(sp.SampleEncode("ABC DEF", -1, 0.5, &output).ok());
@@ -536,7 +535,7 @@ TEST(SentencepieceProcessorTest, SampleEncodeTest) {
   const float prob = 1.0 * freq[0] / (freq[0] + freq[1]);
   EXPECT_NEAR(prob, expected_prob, 0.05);
 
-  auto mock_empty = absl::make_unique<MockModel>();
+  auto mock_empty = std::make_unique<MockModel>();
   mock_empty->SetNBestEncodeResult(kInput, {});
   sp.SetModel(std::move(mock_empty));
   EXPECT_FALSE(sp.SampleEncode("ABC DEF", 10, 0.5, &output).ok());
@@ -578,12 +577,12 @@ TEST(SentencepieceProcessorTest, DecodeTest) {
 
   {
     SentencePieceProcessor sp;
-    auto mock = absl::make_unique<DecodeMockModel>();
+    auto mock = std::make_unique<DecodeMockModel>();
     sp.SetModel(std::move(mock));
 
     const auto normalization_spec = MakeDefaultNormalizerSpec();
     sp.SetNormalizer(
-        absl::make_unique<normalizer::Normalizer>(normalization_spec));
+        std::make_unique<normalizer::Normalizer>(normalization_spec));
 
     SentencePieceText spt;
 
@@ -629,15 +628,15 @@ TEST(SentencepieceProcessorTest, DecodeTest) {
   // unk_surface is not defined.
   {
     SentencePieceProcessor sp;
-    auto proto = absl::make_unique<ModelProto>();
+    auto proto = std::make_unique<ModelProto>();
     sp.Load(std::move(proto)).IgnoreError();
 
-    auto mock = absl::make_unique<DecodeMockModel>();
+    auto mock = std::make_unique<DecodeMockModel>();
     sp.SetModel(std::move(mock));
 
     const auto normalization_spec = MakeDefaultNormalizerSpec();
     sp.SetNormalizer(
-        absl::make_unique<normalizer::Normalizer>(normalization_spec));
+        std::make_unique<normalizer::Normalizer>(normalization_spec));
 
     SentencePieceText spt;
 
@@ -648,16 +647,16 @@ TEST(SentencepieceProcessorTest, DecodeTest) {
 
   {
     SentencePieceProcessor sp;
-    auto proto = absl::make_unique<ModelProto>();
+    auto proto = std::make_unique<ModelProto>();
     proto->mutable_trainer_spec()->set_unk_surface("");
     sp.Load(std::move(proto)).IgnoreError();
 
-    auto mock = absl::make_unique<DecodeMockModel>();
+    auto mock = std::make_unique<DecodeMockModel>();
     sp.SetModel(std::move(mock));
 
     const auto normalization_spec = MakeDefaultNormalizerSpec();
     sp.SetNormalizer(
-        absl::make_unique<normalizer::Normalizer>(normalization_spec));
+        std::make_unique<normalizer::Normalizer>(normalization_spec));
 
     SentencePieceText spt;
 
@@ -668,16 +667,16 @@ TEST(SentencepieceProcessorTest, DecodeTest) {
 
   {
     SentencePieceProcessor sp;
-    auto proto = absl::make_unique<ModelProto>();
+    auto proto = std::make_unique<ModelProto>();
     proto->mutable_trainer_spec()->set_unk_surface("<UNK>");
     sp.Load(std::move(proto)).IgnoreError();
 
-    auto mock = absl::make_unique<DecodeMockModel>();
+    auto mock = std::make_unique<DecodeMockModel>();
     sp.SetModel(std::move(mock));
 
     const auto normalization_spec = MakeDefaultNormalizerSpec();
     sp.SetNormalizer(
-        absl::make_unique<normalizer::Normalizer>(normalization_spec));
+        std::make_unique<normalizer::Normalizer>(normalization_spec));
 
     SentencePieceText spt;
 
@@ -688,18 +687,18 @@ TEST(SentencepieceProcessorTest, DecodeTest) {
 
   {
     SentencePieceProcessor sp;
-    auto proto = absl::make_unique<ModelProto>();
+    auto proto = std::make_unique<ModelProto>();
     proto->mutable_trainer_spec()->set_unk_surface("");
     proto->mutable_normalizer_spec()->set_add_dummy_prefix(false);
     proto->mutable_normalizer_spec()->set_remove_extra_whitespaces(false);
     sp.Load(std::move(proto)).IgnoreError();
 
-    auto mock = absl::make_unique<DecodeMockModel>();
+    auto mock = std::make_unique<DecodeMockModel>();
     sp.SetModel(std::move(mock));
 
     const auto normalization_spec = MakeDefaultNormalizerSpec();
     sp.SetNormalizer(
-        absl::make_unique<normalizer::Normalizer>(normalization_spec));
+        std::make_unique<normalizer::Normalizer>(normalization_spec));
 
     SentencePieceText spt;
 
@@ -746,18 +745,18 @@ TEST(SentencepieceProcessorTest, DummyPrefixDecodeTest) {
 
   {
     SentencePieceProcessor sp;
-    auto proto = absl::make_unique<ModelProto>();
+    auto proto = std::make_unique<ModelProto>();
     proto->mutable_trainer_spec()->set_unk_surface("");
     proto->mutable_normalizer_spec()->set_add_dummy_prefix(true);
     proto->mutable_normalizer_spec()->set_remove_extra_whitespaces(false);
     sp.Load(std::move(proto)).IgnoreError();
 
-    auto mock = absl::make_unique<DecodeMockModel>();
+    auto mock = std::make_unique<DecodeMockModel>();
     sp.SetModel(std::move(mock));
 
     const auto normalization_spec = MakeDefaultNormalizerSpec();
     sp.SetNormalizer(
-        absl::make_unique<normalizer::Normalizer>(normalization_spec));
+        std::make_unique<normalizer::Normalizer>(normalization_spec));
 
     SentencePieceText spt;
 
@@ -768,18 +767,18 @@ TEST(SentencepieceProcessorTest, DummyPrefixDecodeTest) {
 
   {
     SentencePieceProcessor sp;
-    auto proto = absl::make_unique<ModelProto>();
+    auto proto = std::make_unique<ModelProto>();
     proto->mutable_trainer_spec()->set_unk_surface("");
     proto->mutable_normalizer_spec()->set_add_dummy_prefix(true);
     proto->mutable_normalizer_spec()->set_remove_extra_whitespaces(true);
     sp.Load(std::move(proto)).IgnoreError();
 
-    auto mock = absl::make_unique<DecodeMockModel>();
+    auto mock = std::make_unique<DecodeMockModel>();
     sp.SetModel(std::move(mock));
 
     const auto normalization_spec = MakeDefaultNormalizerSpec();
     sp.SetNormalizer(
-        absl::make_unique<normalizer::Normalizer>(normalization_spec));
+        std::make_unique<normalizer::Normalizer>(normalization_spec));
 
     SentencePieceText spt;
 
@@ -833,12 +832,12 @@ TEST(SentencepieceProcessorTest, ByteFallbackDecodeTest) {
   };
 
   SentencePieceProcessor sp;
-  auto mock = absl::make_unique<ByteFallbackDecodeMockModel>();
+  auto mock = std::make_unique<ByteFallbackDecodeMockModel>();
   sp.SetModel(std::move(mock));
 
   const auto normalization_spec = MakeDefaultNormalizerSpec();
   sp.SetNormalizer(
-      absl::make_unique<normalizer::Normalizer>(normalization_spec));
+      std::make_unique<normalizer::Normalizer>(normalization_spec));
 
   {
     const std::vector<std::string> input = {
@@ -1347,7 +1346,7 @@ TEST(SentencePieceProcessorTest, EndToEndTest) {
   // Moves ModelProto.
   {
     SentencePieceProcessor sp;
-    auto moved = absl::make_unique<ModelProto>();
+    auto moved = std::make_unique<ModelProto>();
     const ModelProto *moved_ptr = moved.get();
     *moved = model_proto;
     EXPECT_TRUE(sp.Load(std::move(moved)).ok());
