@@ -57,6 +57,8 @@ ABSL_FLAG(bool, shuffle_input_sentence,
 ABSL_FLAG(int32, seed_sentencepiece_size,
           kDefaultTrainerSpec.seed_sentencepiece_size(),
           "the size of seed sentencepieces");
+ABSL_FLAG(std::string, seed_sentencepieces_file, "",
+          "file to load seed sentencepieces from");
 ABSL_FLAG(double, shrinking_factor, kDefaultTrainerSpec.shrinking_factor(),
           "Keeps top shrinking_factor pieces with respect to the loss");
 ABSL_FLAG(int32, num_threads, kDefaultTrainerSpec.num_threads(),
@@ -221,6 +223,7 @@ int main(int argc, char *argv[]) {
   SetTrainerSpecFromFlag(input_sentence_size);
   SetTrainerSpecFromFlag(shuffle_input_sentence);
   SetTrainerSpecFromFlag(seed_sentencepiece_size);
+  SetTrainerSpecFromFlag(seed_sentencepieces_file);
   SetTrainerSpecFromFlag(shrinking_factor);
   SetTrainerSpecFromFlag(num_threads);
   SetTrainerSpecFromFlag(num_sub_iterations);
