@@ -340,6 +340,10 @@ std::string StrError(int errnum);
 
 std::vector<std::string> StrSplitAsCSV(absl::string_view text);
 
+#ifdef OS_WIN
+std::wstring Utf8ToWide(const absl::string_view input);
+#endif
+
 inline Status OkStatus() { return Status(); }
 
 #define DECLARE_ERROR(FUNC)                                \
@@ -428,5 +432,11 @@ class ThreadPool {
  private:
   std::vector<std::thread> tasks_;
 };
+
+namespace log_domain {
+
+double LogSum(const std::vector<double> &xs);
+
+}  // namespace log_domain
 }  // namespace sentencepiece
 #endif  // UTIL_H_
